/*
 * vim: ts=4 nobackup
 */


#include "TTTMinimaxGamePlayer.H"


using namespace avelino;
using namespace std;


void TTTMinimaxGamePlayer::setOpponentNumber(uint8_t n) {
	this->opponentNumber = n;
}


void TTTMinimaxGamePlayer::internalInit() {
}


uint8_t TTTMinimaxGamePlayer::getSpot() {
	uint8_t ret = 0;
	this->minimax(*this->board, true, ret);
	return ret;
}


int8_t TTTMinimaxGamePlayer::minimax(TTTGameBoard &board, bool me, uint8_t &bestMovement) {
	uint8_t winner = board.getWinner();
	if (board.isFinished() || (winner != 0)) {
		int8_t ret = 0;
		if (winner == this->number)
			ret = 1;
		else if (winner == this->opponentNumber)
			ret = -1;
		return ret;
	}
	else {
		int8_t limit = 0;
		if (me)
			limit = -10;
		else
			limit = 10;
		TTTGameBoard auxBoard;
		uint8_t numMovements = 0;
		for (uint8_t m = board.getFirstAvailableMovement(); m != 0; m = board.getNextAvailableMovement(), numMovements++) {
			auxBoard.copyFrom(board);
			if (me)
				auxBoard.set(m, this->number);
			else
				auxBoard.set(m, this->opponentNumber);
			uint8_t childBestMovement;
			int8_t v = minimax(auxBoard, !me, childBestMovement);
			if (me) {
				if (v > limit) {
					limit = v;
					bestMovement = m;
				}
			}
			else {
				if (v < limit) {
					limit = v;
					bestMovement = m;
				}
			}
		}
		return limit;
	}
}


