/*
 * vim: ts=4 nobackup
 */


#include "util.H"
#include "Timer.H"
#include "MyLedMatrixManager.H"
#include "MyKeyMatrixManager.H"
#ifdef  USE_MINIMAX_GAME_PLAYER
	#include "TTTMinimaxGamePlayer.H"
#else
	#include "TTTRandomGamePlayer.H"
#endif
#include "Game.H"
#include "TTTHumanGamePlayer.H"
#include "TTTLedGameBoard.H"


using namespace std;
using namespace avelino;


#define  GAME_STATUS_PLAYING   1
#define  GAME_STATUS_FINISHED  2


MyLedMatrixManager ledMatrixManager;
MyKeyMatrixManager keyMatrixManager;
#ifdef  USE_MINIMAX_GAME_PLAYER
TTTMinimaxGamePlayer p1;
#else
TTTRandomGamePlayer p1;
#endif
TTTHumanGamePlayer p2;
Game g;
TTTLedGameBoard board;
uint8_t gameStatus;


int main() {
	Timer::init();
	ledMatrixManager.init();
	board.init(ledMatrixManager);
	p1.init(1, board);
#ifdef  USE_MINIMAX_GAME_PLAYER
	p1.setOpponentNumber(2);
#endif
	p2.init(2, board);
	keyMatrixManager.init(p2);
	//g.init(p1, p2, board);   // empieza jugando el Arduino
	g.init(p2, p1, board);     // empieza jugando el jugador
	gameStatus = GAME_STATUS_PLAYING;
	while (true) {
		ledMatrixManager.run();
		if (gameStatus == GAME_STATUS_PLAYING) {
			if (!g.isFinished()) {
				keyMatrixManager.run();
				g.run();
				board.show();
			}
			else
				gameStatus = GAME_STATUS_FINISHED;
		}
		else if (gameStatus == GAME_STATUS_FINISHED) {
			//
		}
	}
	return 0;
}
